from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import validate_slug
from django.utils.translation import gettext_lazy as _
from entangled.forms import EntangledModelForm, EntangledModelFormMixin

from . import (
    _form_registry,
    actions,
    constants,
    get_registered_forms,
    models,
    recaptcha,
    settings,
)
from .entry_model import FormEntry
from .fields import AttributesFormField, ButtonGroup, ChoicesFormField
from .helpers import get_option, mark_safe_lazy


class Noop:
    pass


def mixin_factory(x):
    return Noop


class SimpleFrontendForm(forms.Form):
    takes_request = True

    def __init__(self, *args, **kwargs):
        self._request = kwargs.pop("request")
        if get_option(self, "unique", False) and self._request.user.is_authenticated:
            qs = FormEntry.objects.filter(
                form_user=self._request.user, form_name=get_option(self, "form_name")
            )
            if qs:
                kwargs["initial"] = qs.last().entry_data
        super().__init__(*args, **kwargs)

    def clean(self):
        if get_option(self, "login_required", False):
            if not self._request.user.is_authenticated:
                raise ValidationError(
                    _("Please login before submitting this form."), code="unauthorized"
                )
        return super().clean()

    def save(self):
        results = {}
        form_actions = get_option(self, "form_actions", [])
        for action in form_actions:
            Action = actions.get_action_class(action)
            if Action is not None:
                results[action] = Action().execute(self, self._request)
            else:
                results[action] = _("Action not available any more")
        if not form_actions:
            results[None] = _("No action registered")
        return results


class SelectMultipleActionsWidget(forms.CheckboxSelectMultiple):
    def format_value(self, value):
        import json

        if isinstance(value, str):
            value = json.loads(value.replace("'", '"'))
        return super().format_value(value)


class FormsForm(mixin_factory("Form"), EntangledModelForm):
    """
    Components > "Forms" Plugin
    https://getbootstrap.com/docs/5.1/forms/overview/
    """

    class Meta:
        model = models.Form
        exclude = ()
        untangled_fields = [
            "form_selection",
            "form_name",
            "form_login_required",
            "form_unique",
            "form_floating_labels",
            "form_spacing",
            "form_actions",
            "attributes",
            "captcha_widget",
            "captcha_requirement",
            "captcha_config",
        ]
        entangled_fields = {
            "action_parameters": [],
        }

    form_selection = forms.ChoiceField(
        label=_("Form"),
        required=False,
        initial="",
    )
    form_name = forms.CharField(
        label=_("Form identifier"),
        required=False,
        initial="",
        validators=[
            validate_slug,
        ],
        help_text=_("Slug that allows to uniquely identify forms."),
    )
    form_login_required = forms.BooleanField(
        label=_("Login required to submit form"),
        required=False,
        initial=False,
        help_text=_(
            "To avoid issues with user experience use this type of form only on pages, "
            "which require login."
        ),
    )

    form_unique = forms.BooleanField(
        label=_("User can reopen form"),
        required=False,
        initial=False,
        help_text=_('Requires "Login required" to be checked to work.'),
    )

    form_spacing = forms.ChoiceField(
        label=_("Margin between fields"),
        choices=settings.SPACER_SIZE_CHOICES,
        initial=settings.SPACER_SIZE_CHOICES[len(settings.SPACER_SIZE_CHOICES) // 2][0],
        required=False,
    )

    form_floating_labels = forms.BooleanField(
        label=_("Floating labels"),
        required=False,
        initial=False,
    )

    form_actions = forms.MultipleChoiceField(
        label=_("Actions to be taken after form submission"),
        widget=SelectMultipleActionsWidget(),
        required=False,
    )

    attributes = AttributesFormField()

    captcha_widget = forms.ChoiceField(
        label=_("Captcha widget"),
        required=False,
        initial="v2-invisible" if recaptcha.installed else "",
        choices=settings.EMPTY_CHOICE + recaptcha.CAPTCHA_CHOICES,
        help_text=mark_safe_lazy(
            _(
                'Read more in the <a href="{link}" target="_blank">documentation</a>.'
            ).format(link="https://developers.google.com/recaptcha")
        ),
    )
    captcha_requirement = forms.DecimalField(
        label=_("Minimum score requirement"),
        required=recaptcha.installed,
        initial=0.5,
        min_value=0,
        max_value=1,
        help_text=_(
            "Only for reCaptcha v3: Minimum score required to accept challenge."
        ),
    )
    captcha_config = AttributesFormField(
        label=_("Recaptcha configuration parameters"),
        help_text=mark_safe_lazy(
            _(
                'The reCAPTCHA widget supports several <a href="{attr_link}" target="_blank">data attributes</a> '
                "that customize the behaviour of the widget, such as <code>data-theme</code>, "
                "<code>data-size</code>. "
                'The reCAPTCHA api supports several <a href="{api_link}" target="_blank">parameters</a>. '
                "Add these api parameters as attributes, e.g. <code>hl</code> to set the language."
            ).format(
                attr_link="https://developers.google.com/recaptcha/docs/display#render_param",
                api_link="https://developers.google.com/recaptcha/docs/display#javascript_resource_apijs_parameters",
            )
        ),
    )

    def __init__(self, *args, **kwargs):
        if (
            not _form_registry
            and "instance" in kwargs
            and kwargs["instance"] is not None
        ):
            # remove form_selection data if widget will be hidden
            kwargs["instance"].form_selection = ""
        self._meta.model = models.Form
        super().__init__(*args, **kwargs)
        registered_forms = get_registered_forms()
        available_form_actions = actions.get_registered_actions()
        self.fields["form_selection"].widget = (
            forms.Select() if _form_registry else forms.HiddenInput()
        )
        self.fields["form_selection"].choices = settings.EMPTY_CHOICE + registered_forms
        self.fields["form_actions"].choices = available_form_actions

    def clean(self):
        if self.cleaned_data.get("form_selection", "") == "":
            if not self.cleaned_data.get("form_name", "-"):
                raise ValidationError(
                    {
                        "form_name": _(
                            "Please provide a form name to be able to evaluate form submissions."
                        )
                    },
                    code="incomplete",
                )

        if "form_actions" in self.cleaned_data:
            if (
                self.cleaned_data["form_unique"]
                and actions.SAVE_TO_DB_ACTION not in self.cleaned_data["form_actions"]
            ):
                if actions.SAVE_TO_DB_ACTION:
                    raise ValidationError(
                        {
                            "form_actions": _(
                                'Please select "Save form submission" to allow users to reopen forms.'
                            ),
                            "form_unique": _(
                                'Please select the action "Save form submission" to allow users to reopen forms.'
                            ),
                        }
                    )
                else:
                    raise ValidationError(
                        {
                            "form_unique": _(
                                "No form action to save form contents available. Users will not be "
                                "able to reopen a form."
                            ),
                        }
                    )
        elif not self.cleaned_data["form_selection"]:
            raise ValidationError(
                {
                    "form_actions": _(
                        "At least one action needs to be selected for the form to have an effect."
                    ),
                }
            )

        if (
            self.cleaned_data["form_unique"]
            and not self.cleaned_data["form_login_required"]
        ):
            error = _("Users can only reopen forms if they are logged in. %(remedy)s")
            raise ValidationError(
                {
                    "form_login_required": error
                    % dict(remedy=_("Either enable this.")),
                    "form_unique": _("Or disable this."),
                },
                code="inconsistent",
            )
        return self.cleaned_data

    def is_valid(self):
        valid = super().is_valid()
        return valid


FORBIDDEN_FORM_NAMES = [
    "Meta",
    "get_success_context",
    "form_name",
    "form_user",
    "entry_data",
    "html_header",
] + dir(SimpleFrontendForm(request=None))


def validate_form_name(value):
    if value in FORBIDDEN_FORM_NAMES:
        raise ValidationError(
            _("This name is reserved. Please chose a different one."), code="illegal"
        )


class FormFieldMixin(EntangledModelFormMixin):
    """
    Components > "Forms" Plugin
    https://getbootstrap.com/docs/5.1/forms/overview/
    """

    class Meta:
        entangled_fields = {
            "config": [
                "field_name",
                "field_label",
                "field_placeholder",
                "field_required",
            ]
        }

    field_name = forms.CharField(
        label=_("Field name"),
        help_text=_(
            "Internal field name consisting of letters, numbers, underscores or hyphens"
        ),
        required=True,
        validators=[validate_slug, validate_form_name],
    )
    field_label = forms.CharField(
        label=_("Label"),
        help_text=_(
            "Field label shown to the user describing the entity to be entered"
        ),
        required=False,
    )
    field_placeholder = forms.CharField(
        label=_("Placeholder"),
        help_text=_("Example input shown muted in an empty field"),
        required=False,
    )
    field_required = forms.BooleanField(
        label=_("Required"),
        initial=False,
        required=False,
        help_text=_(
            "If selected form will not accept submissions with with empty data"
        ),
    )


class CharFieldForm(mixin_factory("CharField"), FormFieldMixin, EntangledModelForm):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "min_length",
                "max_length",
            ]
        }

    min_length = forms.IntegerField(
        label=_("Minimum text length"),
        required=False,
        initial=None,
    )
    max_length = forms.IntegerField(
        label=_("Maximum text length"),
        required=False,
        initial=None,
    )


class EmailFieldForm(mixin_factory("EmailField"), FormFieldMixin, EntangledModelForm):
    class Meta:
        model = models.FormField
        entangled_fields = {"config": []}


class UrlFieldForm(mixin_factory("URLField"), FormFieldMixin, EntangledModelForm):
    class Meta:
        model = models.FormField
        entangled_fields = {"config": []}


class DecimalFieldForm(
    mixin_factory("DecimalField"), FormFieldMixin, EntangledModelForm
):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "min_value",
                "max_value",
                "decimal_places",
            ]
        }

    min_value = forms.DecimalField(
        label=_("Minimum value"),
        required=False,
        initial=None,
    )
    max_value = forms.DecimalField(
        label=_("Maximum value"), required=False, initial=None
    )
    decimal_places = forms.IntegerField(
        label=_("Decimal places"),
        required=False,
        initial=None,
        min_value=0,
    )


class IntegerFieldForm(
    mixin_factory("IntegerField"), FormFieldMixin, EntangledModelForm
):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "min_value",
                "max_value",
            ]
        }

    min_value = forms.IntegerField(
        label=_("Minimum value"),
        required=False,
        initial=None,
    )
    max_value = forms.IntegerField(
        label=_("Maximum value"), required=False, initial=None
    )


class TextareaFieldForm(
    mixin_factory("TextareaField"), FormFieldMixin, EntangledModelForm
):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "min_length",
                "max_length",
                "field_rows",
            ]
        }

    field_rows = forms.IntegerField(
        label=_("Rows"),
        min_value=1,
        max_value=40,
        initial=10,
        help_text=_("Defines the vertical size of the text area in number of rows."),
    )
    min_length = forms.IntegerField(
        label=_("Minimum text length"),
        required=False,
        initial=None,
    )
    max_length = forms.IntegerField(
        label=_("Maximum text length"),
        required=False,
        initial=None,
    )


class DateFieldForm(mixin_factory("DateField"), FormFieldMixin, EntangledModelForm):
    class Meta:
        model = models.FormField
        entangled_fields = {"config": []}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["field_placeholder"].help_text = _("Not visible on most browsers.")


class DateTimeFieldForm(
    mixin_factory("DateTimeField"), FormFieldMixin, EntangledModelForm
):
    class Meta:
        model = models.FormField
        entangled_fields = {"config": []}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["field_placeholder"].help_text = _("Not visible on most browsers.")


class TimeFieldForm(mixin_factory("TimeField"), FormFieldMixin, EntangledModelForm):
    class Meta:
        model = models.FormField
        entangled_fields = {"config": []}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["field_placeholder"].help_text = _("Not visible on most browsers.")


class SelectFieldForm(mixin_factory("SelectField"), FormFieldMixin, EntangledModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if "instance" in kwargs and kwargs["instance"] is not None:
            self.fields["field_choices"].initial = kwargs["instance"].get_choices()

    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "field_select",
            ]
        }
        untangled_fields = ("field_choices",)

    field_select = forms.ChoiceField(
        label=_("Selection type"),
        required=True,
        choices=constants.CHOICE_FIELDS,
        widget=ButtonGroup(
            attrs=dict(property="text", label_class="btn-outline-secondary")
        ),
    )
    field_choices = ChoicesFormField(
        required=True,
    )

    def clean(self):
        if (
            self.cleaned_data.get("field_required", False)
            and self.cleaned_data.get("field_select", "") == "checkbox"
        ):
            raise ValidationError(
                {
                    "field_select": mark_safe_lazy(
                        _(
                            "For a required multiple choice fild select the <b>list</b> selection type."
                        )
                    ),
                    "field_required": mark_safe_lazy(
                        _("Checkbox multiple choice field <b>must not be required</b>.")
                    ),
                }
            )
        return self.cleaned_data


class ChoiceForm(EntangledModelForm):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "value",
                "verbose",
            ]
        }

    value = forms.CharField(
        label=_("Value"),
        required=True,
        help_text=_("Stored in database if the choice is selected."),
    )
    verbose = forms.CharField(
        label=_("Display text"),
        required=True,
        help_text=_("Representation of choice displayed to the user."),
    )


class BooleanFieldForm(
    mixin_factory("BooleanField"), FormFieldMixin, EntangledModelForm
):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "field_as_switch",
            ]
        }

    field_as_switch = forms.BooleanField(
        label=_("Layout"),
        initial=False,
        required=False,
        widget=ButtonGroup(
            choices=((False, _("Checkbox")), (True, _("Switch"))),
            attrs=dict(property="text", label_class="btn-outline-secondary"),
        ),
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["field_required"].help_text = _(
            "If checked, the form can only be submitted if the "
            "checkbox is checked or the switch set to on."
        )


class SubmitButtonForm(
    mixin_factory("SubmitButton"), FormFieldMixin, EntangledModelForm
):
    class Meta:
        model = models.FormField
        entangled_fields = {
            "config": [
                "submit_cta",
            ]
        }

    submit_cta = forms.CharField(
        label=_("Button label"),
        initial=_("Submit"),
        required=False,
    )



# Generated by Django 3.2 on 2023-01-29 19:50

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_form_builder', '0002_alter_form_cmsplugin_ptr_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='form',
            name='tag_type',
        ),
        migrations.RemoveField(
            model_name='formfield',
            name='tag_type',
        ),
        migrations.AddField(
            model_name='form',
            name='action_parameters',
            field=models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder, null=True),
        ),
        migrations.AlterField(
            model_name='form',
            name='captcha_widget',
            field=models.CharField(blank=True, choices=[('', '-----')], default='', help_text='Read more in the <a href="https://developers.google.com/recaptcha" target="_blank">documentation</a>.', max_length=16, verbose_name='captcha widget'),
        ),
    ]



import json
from urllib.parse import urlencode

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.http import Http404, JsonResponse
from django.template.context_processors import csrf
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _
from django.views.generic.edit import FormMixin
from sekizai.context import SekizaiContext

from djangocms_form_builder import settings

from .. import forms, models, recaptcha
from ..actions import ActionMixin
from ..forms import SimpleFrontendForm
from ..helpers import get_option, insert_fields, mark_safe_lazy

SAME_PAGE_REDIRECT = "result"


class CMSAjaxBase(CMSPluginBase):
    def ajax_post(self, request, instance, parameter):
        return JsonResponse({})

    def ajax_get(self, request, instance, parameter):
        return JsonResponse({})


class AjaxFormMixin(FormMixin):
    form_class = None
    replace = True
    request = None
    instance = None
    parameter = {}
    template_name = None

    def json_return(self, errors, result, redirect, content):
        return JsonResponse(
            {
                "result": result,
                "redirect": redirect,
                "errors": errors,
                "field_errors": {},
                "content": content,
            }
        )

    def form_valid(self, form):
        # Execute save method
        save = getattr(form, "save", None)
        if callable(save):
            result = form.save()
        # Identify redirect
        redirect = get_option(form, "redirect", None)
        if isinstance(redirect, str):
            try:
                redirect = reverse(redirect)
            except NoReverseMatch:
                pass
        elif hasattr(redirect, "get_absolute_url"):
            redirect = redirect.get_absolute_url()

        get_success_context = "get_success_context"
        render_success = "render_success"
        if hasattr(form, "slug"):
            get_success_context += "_" + form.slug
            render_success += "_" + form.slug

        if get_option(form, render_success, None):
            context = SekizaiContext(
                {
                    "form": form,
                    "instance": self.instance,
                    "request": self.request,
                    "get_str": urlencode(
                        {x: y for x, y in self.request.POST.items() if "csrf" not in x}
                    ),
                }
            )
            if hasattr(form, get_success_context):
                get_success_context = getattr(form, get_success_context)
                context.update(get_success_context(self.request, self.instance, form))
            print(context)
            errors, result, redir, content = (
                [],
                context.get("result", "success"),
                "" if self.replace else "result",
                render_to_string(
                    get_option(form, render_success), context.flatten(), self.request
                ),
            )
        elif redirect:
            errors, result, redir, content = (
                [],
                "success",
                redirect,
                "",
            )
        else:
            errors, result, redir, content = (
                [_("No content in response from")],
                "error",
                "",
                "",
            )
        redirect = redirect or redir
        return JsonResponse(
            {
                "result": result,
                "redirect": redirect,
                "errors": errors,
                "field_errors": {},
                "content": content,
            }
        )

    def form_invalid(self, form):
        return JsonResponse(
            {
                "result": "invalid form",
                "errors": form.non_field_errors(),
                "field_errors": {
                    key + str(self.instance.id): value
                    for key, value in form.errors.items()
                },
                "html": form.render(context=csrf(self.request))
                if hasattr(form, "render")
                else "",  # Kills reCAPTCHA
            }
        )

    def get_form_class(self, slug=None):
        if hasattr(self, "form_classes") and isinstance(self.form_classes, list):
            slug = slug or getattr(self, "parameter", {}).get("s", "")
            if not slug:
                return self.form_classes[0]
            for cls in self.form_classes:
                if getattr(cls, "slug", None) == slug:
                    return cls
            raise Http404
        return super().get_form_class()

    def get_initial(self, slug=None):
        slug = slug or getattr(self, "parameter", {}).get("s", "")
        initial = "initial_" + slug
        if hasattr(self, initial):
            return getattr(self, initial).copy()
        return super().get_initial()

    def get_form_kwargs(self, slug=None):
        """Return the keyword arguments for instantiating the form."""
        kwargs = {
            "initial": self.get_initial(slug),
            "prefix": self.get_prefix(),
            "label_suffix": "",
        }

        if self.request.method in ("POST", "PUT"):
            kwargs.update(
                {
                    "data": self.request.POST,
                    "files": self.request.FILES,
                }
            )
        return kwargs

    def get_ajax_form(self, slug=None):
        form_class = self.get_form_class(slug)
        if form_class:
            if getattr(form_class, "takes_request", False):
                form = form_class(request=self.request, **self.get_form_kwargs(slug))
            else:
                form = form_class(**self.get_form_kwargs(slug))
            if self.instance:
                for field in form.base_fields:
                    form.fields[field].widget.attrs.update(
                        {"id": (field or "") + str(self.instance.id)}
                    )
            return form
        return None

    def ajax_post(self, request, instance, parameter=None):
        if parameter is None:
            parameter = {}
        self.request = request
        self.instance = instance
        self.parameter = parameter

        form = self.get_ajax_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def ajax_get(self, request, instance, parameter=None):
        if parameter is None:
            parameter = {}
        self.request = request
        self.instance = instance
        self.parameter = parameter
        context = self.get_context_data(**parameter)
        errors, redirect, content = (
            [],
            "",
            render_to_string(self.template_name, context.flatten(), self.request),
        )
        return JsonResponse(
            {
                "result": (
                    ("result" if redirect == "result" else "success")
                    if errors == []
                    else "error"
                ),
                "redirect": redirect,
                "errors": errors,
                "field_errors": {},
                "content": content,
            }
        )


class CMSAjaxForm(AjaxFormMixin, CMSAjaxBase):
    def get_form(self, request, *args, **kwargs):
        """
        get_form needs to come from CMSAjaxBase and NOT from AjaxFormMixin
        for admin to work
        """
        return super(CMSAjaxBase, self).get_form(request, *args, **kwargs)

    def set_context(self, context, instance, placeholder):
        return {}

    def render(self, context, instance, placeholder):
        self.instance = instance
        self.request = context["request"]
        form = self.get_ajax_form()
        context.update(self.set_context(context, instance, placeholder))
        context["form_counter"] = context.get("form_counter", 0) + 1
        context.update({
            "instance": instance,
            "form": form,
            "uid": f"{instance.id}{getattr(form, 'slug', '')}-{context['form_counter']}",
        })
        return context


@plugin_pool.register_plugin
class FormPlugin(ActionMixin, CMSAjaxForm):
    name = _("Form")
    model = models.Form

    form = forms.FormsForm
    render_template = f"djangocms_form_builder/{settings.framework}/form.html"
    change_form_template = "djangocms_frontend/admin/base.html"
    allow_children = True

    fieldsets = [
        (
            None,
            {
                "fields": [
                    "form_selection",
                    "form_name",
                    (
                        "form_login_required",
                        "form_unique",
                    ),
                    "form_floating_labels",
                    "form_spacing",
                ],
            },
        ),
    ]

    cache_parent_classes = False

    @classmethod
    def get_parent_classes(cls, slot, page, instance=None):
        """Only valid if not inside form"""
        parent = instance
        while parent is not None:
            if parent.plugin_type == cls.__name__:
                return [""]
            parent = parent.parent
        return super().get_parent_classes(slot, page, instance)

    def get_fieldsets(self, request, obj=None):
        fieldsets = super().get_fieldsets(request, obj)
        if obj is None or not obj.form_selection:  # No Actions if a Django form has been selected
            fieldsets = insert_fields(
                fieldsets,
                ("form_actions",),
                block=None,
                position=1,
                blockname=_("Actions"),
                blockattrs={"classes": ("collapse", "action-auto-hide")},
            )
        if recaptcha.installed:
            return insert_fields(
                fieldsets,
                ("captcha_widget", "captcha_requirement", "captcha_config"),
                block=None,
                position=1,
                blockname=_("Captcha"),
                blockattrs={}
                if recaptcha.keys_available
                else dict(
                    description=mark_safe_lazy(
                        _(
                            '<blockquote style="color: var(--error-fg);">Please get a public and secret '
                            'key from <a href="{key_link}" target="_blank">Google</a> '
                            "and ensure that they are available through django settings "
                            "<code>RECAPTCHA_PUBLIC_KEY</code> and <code>RECAPTCHA_PRIVATE_KEY</code>. "
                            "Without these keys captcha protection will not work.</blockquote>"
                        ).format(key_link="https://developers.google.com/recaptcha")
                    )
                ),
            )
        return fieldsets

    def get_form_class(self, slug=None):
        """Retrieve or create form for this plugin"""
        if self.instance.child_plugin_instances is None:  # not set if in ajax_post
            self.instance.child_plugin_instances = [
                child.get_plugin_instance()[0] for child in self.instance.get_children()
            ]
        if self.instance.child_plugin_instances:
            return self.create_form_class_from_plugins()
        if self.instance.form_selection:
            return forms._form_registry.get(self.instance.form_selection, None)
        return None

    def create_form_class_from_plugins(self):
        def traverse(instance):
            """Recursively traverse children to identify form fields (by them having a method called
            "get_form_field" """
            nonlocal fields
            if hasattr(instance, "get_form_field"):
                name, field = instance.get_form_field()
                fields[name] = field
            if (
                instance.child_plugin_instances is None
            ):  # children already fetched from db?
                instance.child_plugin_instances = [
                    child.get_plugin_instance()[0] for child in instance.get_children()
                ]
            for child in instance.child_plugin_instances:
                traverse(child)

        fields = {}
        traverse(self.instance)

        # Add recaptcha field if necessary
        if recaptcha.installed and self.instance.captcha_widget:
            fields[recaptcha.field_name] = recaptcha.get_recaptcha_field(self.instance)

        # Collect meta options for Meta class
        meta_options = dict(form_name=self.instance.form_name)
        if self.instance.form_floating_labels:
            meta_options["floating_labels"] = True
        meta_options[
            "field_sep"
        ] = f'{self.instance.form_spacing}'
        meta_options[
            "redirect"
        ] = SAME_PAGE_REDIRECT  # Default behavior: redirect to same page
        meta_options["login_required"] = self.instance.form_login_required
        meta_options["unique"] = self.instance.form_unique
        form_actions = self.instance.form_actions or "[]"
        meta_options["form_actions"] = json.loads(form_actions.replace("'", '"'))
        meta_options["form_parameters"] = getattr(self.instance, "action_parameters", {})

        fields["Meta"] = type("Meta", (), dict(
            options=meta_options,
            verbose_name=self.instance.form_name.replace("-", " ").replace("_", " ").capitalize(),
        ))  # Meta class with options and verbose name

        return type(
            "FrontendAutoForm",
            (SimpleFrontendForm,),
            fields,
        )

    def render(self, context, instance, placeholder):
        self.instance = instance
        context["RECAPTCHA_PUBLIC_KEY"] = recaptcha.RECAPTCHA_PUBLIC_KEY
        return super().render(context, instance, placeholder)

    def __str__(self):
        return force_str(super().__str__())



import decimal

from cms.models import CMSPlugin
from django import forms
from django.core.serializers.json import DjangoJSONEncoder
from django.core.validators import validate_slug
from django.db import models
from django.forms.widgets import Input
from django.utils.html import conditional_escape, mark_safe
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from . import recaptcha, settings
from .entry_model import FormEntry  # NoQA
from .fields import AttributesField
from .helpers import coerce_decimal, mark_safe_lazy

MAX_LENGTH = 256


class Form(CMSPlugin):
    class Meta:
        verbose_name = _("Form")

    form_selection = models.CharField(
        verbose_name=_("Form"),
        max_length=MAX_LENGTH,
        blank=True,
        default="",
    )
    form_name = models.CharField(
        verbose_name=_("Form name"),
        max_length=MAX_LENGTH,
        blank=True,
        default="",
        validators=[
            validate_slug,
        ],
    )
    form_login_required = models.BooleanField(
        verbose_name=_("Login required to submit form"),
        blank=True,
        default=False,
        help_text=_(
            "To avoid issues with user experience use this type of form only on pages, "
            "which require login."
        ),
    )
    form_unique = models.BooleanField(
        verbose_name=_("User can reopen form"),
        default=False,
        help_text=_('Requires "Login required" to be checked to work.'),
    )
    form_floating_labels = models.BooleanField(
        verbose_name=_("Floating labels"),
        default=False,
    )
    form_spacing = models.CharField(
        verbose_name=_("Margin between fields"),
        max_length=16,
    )

    form_actions = models.CharField(
        verbose_name=_("Actions to be taken after form submission"),
        blank=True,
        max_length=4 * MAX_LENGTH,
    )

    action_parameters = models.JSONField(
        default=dict,
        encoder=DjangoJSONEncoder,
        blank=True,
        null=True,
    )

    attributes = AttributesField()

    captcha_widget = models.CharField(
        verbose_name=_("captcha widget"),
        max_length=16,
        blank=True,
        default="v2-invisible" if recaptcha.installed else "",
        choices=settings.EMPTY_CHOICE + recaptcha.CAPTCHA_CHOICES,
        help_text=mark_safe_lazy(
            _(
                'Read more in the <a href="{link}" target="_blank">documentation</a>.'
            ).format(link="https://developers.google.com/recaptcha")
        ),
    )
    captcha_requirement = models.DecimalField(
        verbose_name=_("Minimum score requirement"),
        null=not recaptcha.installed,
        decimal_places=2,
        max_digits=3,
        default=0.5,
        help_text=_(
            "Only for reCaptcha v3: Minimum score required to accept challenge."
        ),
    )
    captcha_config = AttributesField(
        verbose_name=_("Recaptcha configuration parameters"),
        help_text=mark_safe_lazy(
            _(
                'The reCAPTCHA widget supports several <a href="{attr_link}" target="_blank">data attributes</a> '
                "that customize the behaviour of the widget, such as <code>data-theme</code>, "
                "<code>data-size</code>. "
                'The reCAPTCHA api supports several <a href="{api_link}" target="_blank">parameters</a>. '
                "Add these api parameters as attributes, e.g. <code>hl</code> to set the language."
            ).format(
                attr_link="https://developers.google.com/recaptcha/docs/display#render_param",
                api_link="https://developers.google.com/recaptcha/docs/display#javascript_resource_apijs_parameters",
            )
        ),
    )

    def get_short_description(self):
        return f"({self.form_name})" if self.form_name else "<unnamed>"

    def __str__(self):
        return f"{self.__class__.__name__} ({self.id})"


class FormField(CMSPlugin):
    """
    Generic plugin model to store all form items. Plugin and field-specific information is stored in a JSON field
    called "config".
    """

    class Meta:
        verbose_name = gettext("Form field item")

    ui_item = models.CharField(max_length=30)
    config = models.JSONField(default=dict, encoder=DjangoJSONEncoder)

    def __init__(self, *args, **kwargs):
        self._additional_classes = []
        super().__init__(*args, **kwargs)

    def __getattr__(self, item):
        """Makes properties of plugin config available as plugin properties."""
        if (
            item[0] != "_" and item in self.config
        ):  # Avoid infinite recursion trying to get .config from db
            return self.config[item]
        return super().__getattribute__(item)

    def __str__(self):
        if "__str__" in self.config:
            return self.config["__str__"]
        return f"{gettext(self.ui_item)} ({str(self.pk)})"

    def add_classes(self, *args):
        for arg in args:
            if arg:
                self._additional_classes += arg.split() if isinstance(arg, str) else arg

    def add_attribute(self, attr, value=None):
        attrs = self.config.get("attributes", {})
        attrs.update({attr: value})
        self.config["attributes"] = attrs

    def get_attributes(self):
        attributes = self.config.get("attributes", {})
        classes = set(attributes.get("class", "").split())
        classes.update(self._additional_classes)
        if classes:
            attributes["class"] = " ".join(classes)
        parts = (
            f'{item}="{conditional_escape(value)}"' if value else f"{item}"
            for item, value in attributes.items()
        )
        return mark_safe(" " + " ".join(parts)) if parts else ""

    def save(self, *args, **kwargs):
        self.ui_item = self.__class__.__name__
        return super().save(*args, **kwargs)

    def initialize_from_form(self, form=None):
        """Populates the config JSON field based on initial values provided by the fields of form"""
        if form is None:
            form = self.get_plugin_class().form
        if isinstance(form, type):  # if is class
            form = form()  # instantiate
        entangled_fields = getattr(
            getattr(form, "Meta", None), "entangled_fields", {}
        ).get("config", ())
        for field in entangled_fields:
            self.config.setdefault(
                field, {} if field == "attributes" else form[field].initial or ""
            )
        return self

    def get_short_description(self):
        label = self.config.get("field_label", "")
        return f"{label} ({self.config.get('field_name')})"


class CharField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Character field")

    def get_form_field(self):
        return self.field_name, forms.CharField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=forms.TextInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", ""))
            ),
        )


class EmailField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Email field")

    def get_form_field(self):
        return self.field_name, forms.EmailField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=forms.EmailInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", ""))
            ),
        )


class UrlField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("URL field")

    def get_form_field(self):
        return self.field_name, forms.URLField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=forms.URLInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", ""))
            ),
        )


class DecimalField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Decimal field")

    class NumberInput(forms.NumberInput):
        def __init__(self, **kwargs):
            self.decimal_places = kwargs.pop("decimal_places", None)
            super().__init__(**kwargs)

        def format_value(self, value):
            if value is None:
                return ""
            value = str(value)
            if "." in value:
                left, right = value.rsplit(".", 1)
            else:
                left, right = value, ""
            if self.decimal_places == 0:
                return left
            right = (right + self.decimal_places * "0")[: self.decimal_places]
            return super().format_value(".".join((left, right)))

    class StrDecimalField(forms.DecimalField):
        def clean(self, value):
            value = super().clean(value)
            if isinstance(value, decimal.Decimal):
                value = str(value)
            return value

    def get_form_field(self):
        return self.field_name, DecimalField.StrDecimalField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            min_value=coerce_decimal(self.config.get("min_value", None)),
            max_value=coerce_decimal(self.config.get("max_value", None)),
            decimal_places=self.config.get("decimal_places", None),
            widget=DecimalField.NumberInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", "")),
                decimal_places=self.config.get("decimal_places", None),
            ),
        )


class IntegerField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Integer field")

    def get_form_field(self):
        return self.field_name, forms.IntegerField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=DecimalField.NumberInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", "")),
            ),
        )


class TextareaField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Text field")

    def get_form_field(self):
        return self.field_name, forms.CharField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=forms.Textarea(
                attrs=dict(
                    rows=self.config.get("field_rows", 10),
                    placeholder=self.config.get("field_placeholder", ""),
                    style="height: inherit;",
                )
            ),
        )


class DateField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Date field")

    class DateInput(forms.DateInput):
        input_type = "date"

    def get_form_field(self):
        return self.field_name, forms.DateField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=DateField.DateInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", "")),
            ),
        )


class DateTimeField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Date field")

    class DateTimeField(forms.DateTimeField):
        def prepare_value(self, value):
            if isinstance(value, str):
                from django.utils import dateparse

                return super().prepare_value(dateparse.parse_datetime(value))
            return super().prepare_value(value)

    class DateTimeInput(forms.DateTimeInput):
        input_type = "datetime-local"

    def get_form_field(self):
        return self.field_name, DateTimeField.DateTimeField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=DateTimeField.DateTimeInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", "")),
            ),
        )


class TimeField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Date field")

    class TimeInput(forms.TimeInput):
        input_type = "time"

    def get_form_field(self):
        return self.field_name, forms.TimeField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=TimeField.TimeInput(
                attrs=dict(placeholder=self.config.get("field_placeholder", "")),
            ),
        )


class Select(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Select")

    _choices = None
    no_selection = [("", _("No selection"))]

    def get_choices(self):
        if self._choices is None:
            descendants = self.get_children().order_by("position")
            self._choices = []
            for child in descendants:
                instance = child.djangocms_form_builder_formfield
                self._choices.append(
                    (instance.config["value"], instance.config["verbose"])
                )
        return self._choices

    def get_form_field(self):
        multiple_choice = self.config.get("field_select", "") in (
            "multiselect",
            "checkbox",
        )
        field = forms.MultipleChoiceField if multiple_choice else forms.ChoiceField
        required = self.config.get("field_required", False)
        choices = self.get_choices()
        if not required and not multiple_choice:
            choices = self.no_selection + choices
        widget_choice = self.config.get("field_select", "")
        if widget_choice == "select":
            widget = forms.Select()
        elif widget_choice == "radio":
            widget = forms.RadioSelect()
        elif widget_choice == "multiselect":
            widget = forms.SelectMultiple(attrs=dict(style="min-height: 6em;"))
        else:
            widget = forms.CheckboxSelectMultiple()

        return self.field_name, field(
            label=self.config.get("field_label", ""),
            required=required,
            choices=choices,
            widget=widget,
        )


class Choice(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Choice")

    def get_short_description(self):
        return f'{self.config.get("verbose", "-")} ("{self.config.get("value", "")}")'


class SwitchInput(forms.CheckboxInput):
    pass


class BooleanField(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Boolean field")

    def get_form_field(self):
        return self.field_name, forms.BooleanField(
            label=self.config.get("field_label", ""),
            required=self.config.get("field_required", False),
            widget=SwitchInput()
            if self.config.get("field_as_switch", False)
            else forms.CheckboxInput(),
        )


class FormSubmitButton(forms.Field):
    widget = Input(attrs=dict(type="submit"))


class SubmitButton(FormField):
    class Meta:
        proxy = True
        verbose_name = _("Submit button")



import hashlib

from django import forms
from django.apps import apps
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import mail_admins, send_mail
from django.core.validators import EmailValidator
from django.template import TemplateDoesNotExist
from django.template.loader import render_to_string
from django.utils.html import strip_tags
from django.utils.translation import gettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLFormField
from entangled.forms import EntangledModelFormMixin

from . import models
from .entry_model import FormEntry
from .helpers import get_option, insert_fields
from .settings import MAIL_TEMPLATE_SETS

_action_registry = {}


def get_registered_actions():
    """Creates a tuple for a ChoiceField to select form"""
    result = tuple(
        (hash, action_class.verbose_name)
        for hash, action_class in _action_registry.items()
    )
    return result if result else ((_("No actions registered"), ()),)


def register(action_class):
    """Function to call or decorator for an Action class to make it available for the plugin"""

    if not issubclass(action_class, FormAction):
        raise ImproperlyConfigured(
            "djangocms_form_builder.actions.register only "
            "accepts subclasses of djangocms_form_builder.actions.FormAction"
        )
    if not action_class.verbose_name:
        raise ImproperlyConfigured(
            "FormActions need to have a verbose_name property to be registered",
        )
    hash = hashlib.sha1(action_class.__name__.encode("utf-8")).hexdigest()
    _action_registry.update({hash: action_class})
    return action_class


def unregister(action_class):
    hash = hashlib.sha1(action_class.__name__.encode("utf-8")).hexdigest()
    if hash in _action_registry:
        del _action_registry[hash]
    return action_class


def get_action_class(action):
    return _action_registry.get(action, None)


class ActionMixin:
    """Adds action form elements to Form plugin admin"""
    def get_form(self, request, *args, **kwargs):
        """Creates new form class based adding the actions as mixins"""
        return type(
            "FormActionAdminForm",
            (self.form, *_action_registry.values()),
            {}
        )

    def get_fieldsets(self, request, obj=None):
        fieldsets = super().get_fieldsets(request, obj)
        for action in _action_registry.values():
            new_fields = list(action.declared_fields.keys())
            if new_fields:
                hash = hashlib.sha1(action.__name__.encode("utf-8")).hexdigest()
                fieldsets = insert_fields(
                    fieldsets,
                    new_fields,
                    block=None,
                    position=-1,
                    blockname=action.verbose_name,
                    blockattrs=dict(classes=(hash, 'action-hide')),
                )
        return fieldsets


class FormAction(EntangledModelFormMixin):
    class Meta:
        entangled_fields = {"action_parameters": []}
        model = models.Form
        exclude = ()

    class Media:
        js = ("djangocms_form_builder/js/actions_form.js",)
        css = {"all": ("djangocms_form_builder/css/actions_form.css",)}

    verbose_name = None

    def execute(self, form, request):
        raise NotImplementedError()

    @staticmethod
    def get_parameter(form, param):
        return (get_option(form, "form_parameters") or {}).get(param, None)


@register
class SaveToDBAction(FormAction):
    verbose_name = _("Save form submission")

    def execute(self, form, request):
        if get_option(form, "unique", False) and get_option(
            form, "login_required", False
        ):
            keys = {
                "form_name": get_option(form, "form_name"),
                "form_user": request.user,
            }
            defaults = {}
        else:
            keys = {}
            defaults = {
                "form_name": get_option(form, "form_name"),
                "form_user": None if request.user.is_anonymous else request.user
            }
        defaults.update(
            {
                "entry_data": form.cleaned_data,
                "html_headers": dict(
                    user_agent=request.headers["User-Agent"],
                    referer=request.headers["Referer"],
                ),
            }
        )
        if keys:  # update_or_create only works if at least one key is given
            try:
                FormEntry.objects.update_or_create(**keys, defaults=defaults)
            except FormEntry.MultipleObjectsReturned:  # Delete outdated objects
                FormEntry.objects.filter(**keys).delete()
                FormEntry.objects.create(**keys, **defaults)
        else:
            FormEntry.objects.create(**defaults), True


SAVE_TO_DB_ACTION = next(iter(_action_registry)) if _action_registry else None


def validate_recipients(value):
    recipients = value.split()
    for recipient in recipients:
        EmailValidator(message=_("Please replace \"%s\" by a valid email address.") % recipient)(recipient)


@register
class SendMailAction(FormAction):
    class Meta:
        entangled_fields = {
            "action_parameters": [
                "sendemail_recipients",
                "sendemail_template",
            ]
        }

    verbose_name = _("Send email")
    from_mail = None
    template = "djangocms_form_builder/actions/mail.html"
    subject = _("%(form_name)s form submission")

    sendemail_recipients = forms.CharField(
        label=_("Mail recipients"),
        required=False,
        initial="",
        validators=[
            validate_recipients,
        ],
        help_text=_("Space or newline separated list of email addresses."),
        widget=forms.Textarea,
    )

    sendemail_template = forms.ChoiceField(
        label=_("Mail template set"),
        required=True,
        initial=MAIL_TEMPLATE_SETS[0][0],
        choices=MAIL_TEMPLATE_SETS,
        widget=forms.Select if len(MAIL_TEMPLATE_SETS) > 1 else forms.HiddenInput,
    )

    def execute(self, form, request):
        recipients = (self.get_parameter(form, "sendemail_recipients") or [])
        template_set = self.get_parameter(form, "sendemail_template") or "default"
        context = dict(
            cleaned_data=form.cleaned_data,
            form_name=getattr(form.Meta, "verbose_name", ""),
            user=request.user,
            user_agent=request.headers["User-Agent"],
            referer=request.headers["Referer"],
        )

        html_message = render_to_string(f"djangocms_form_builder/mails/{template_set}/mail_html.html", context)
        try:
            message = render_to_string(f"djangocms_form_builder/mails/{template_set}/mail.txt", context)
        except TemplateDoesNotExist:
            message = strip_tags(html_message)
        try:
            subject = render_to_string(f"djangocms_form_builder/mails/{template_set}/subject.txt", context)
        except TemplateDoesNotExist:
            subject = self.subject % dict(form_name=context["form_name"])
        if not recipients:
            mail_admins(
                subject,
                message,
                fail_silently=True,
                html_message=html_message,
            )
        else:
            send_mail(
                subject,
                message,
                recipients,
                self.from_mail,
                fail_silently=True,
                html_message=html_message,
            )


@register
class SuccessMessageAction(FormAction):
    verbose_name = _("Success message")

    class Meta:
        entangled_fields = {
            "action_parameters": [
                "submitmessage_message",
            ]
        }

    submitmessage_message = HTMLFormField(
        label=_("Message"),
        required=True,
        initial=_("<p>Thank you for your submission.</p>"),
    )

    def execute(self, form, request):
        message = self.get_parameter(form, "submitmessage_message")
        form.get_success_context = lambda *args, **kwargs: {"message": message}
        form.Meta.options["render_success"] = "djangocms_form_builder/actions/submit_message.html"
        form.Meta.options["redirect"] = None


if apps.is_installed("djangocms_link"):
    from djangocms_link.fields import LinkFormField
    from djangocms_link.helpers import get_link

    @register
    class RedirectAction(FormAction):
        verbose_name = _("Redirect after submission")

        class Meta:
            entangled_fields = {
                "action_parameters": [
                    "redirect_link",
                ]
            }

        redirect_link = LinkFormField(
            label=_("Link"),
            required=True,
        )

        def execute(self, form, request):
            form.Meta.options["redirect"] = get_link(self.get_parameter(form, "redirect_link"))



import hashlib

try:
    from django.utils.translation import gettext_lazy as _
except ModuleNotFoundError:
    _ = lambda x: x  # noqa: E731


__version__ = "0.3.0"

_form_registry = {}


def verbose_name(form_class):
    """returns the verbose_name property of a Meta class if present or else
    splits the camel-cased form class name"""
    if hasattr(form_class, "Meta") and hasattr(form_class.Meta, "verbose_name"):
        return getattr(form_class.Meta, "verbose_name")  # noqa
    class_name = form_class.__name__.rsplit(".", 1)[-1]
    from re import finditer

    matches = finditer(
        ".+?(?:(?<=[a-z])(?=[A-Z])|(?<=[A-Z])(?=[A-Z][a-z])|$)", class_name
    )
    return " ".join(m.group(0) for m in matches)


def get_registered_forms():
    """Creates a tuple for a ChoiceField to select form"""
    result = tuple(
        (hash, verbose_name(form_class)) for hash, form_class in _form_registry.items()
    )
    return result if result else ((_("No forms registered"), ()),)


def register_with_form_builder(form_class):
    """Function to call or decorator for a Form class to make it available for the plugin"""
    hash = hashlib.sha1(form_class.__name__.encode("utf-8")).hexdigest()
    _form_registry.update({hash: form_class})
    return form_class


__all__ = ["register_with_form_builder", "get_registered_forms"]



import decimal

from django import forms
from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.translation import gettext_lazy as _
from entangled.forms import EntangledModelForm


class CSValues(forms.CharField):
    class CSVWidget(forms.TextInput):
        def format_value(self, value):
            return ", ".join(value)

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("widget", CSValues.CSVWidget())
        super().__init__(*args, **kwargs)

    def to_python(self, value):
        value = value.split(",")
        value = list(map(lambda x: x.strip(), value))
        return value


class FormEntry(models.Model):
    class Meta:
        verbose_name = _("Form entry")
        verbose_name_plural = _("Form entries")

    form_name = models.SlugField(
        verbose_name=_("Form"),
        blank=False,
    )
    form_user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("User"),
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    entry_data = models.JSONField(
        default=dict,
        blank=True,
        encoder=DjangoJSONEncoder,
    )
    html_headers = models.JSONField(
        default=dict,
        blank=True,
    )
    entry_created_at = models.DateTimeField(auto_now_add=True)
    entry_updated_at = models.DateTimeField(auto_now=True)

    def get_admin_form(self):
        entangled_fields = []
        fields = {}
        for key, value in self.entry_data.items():
            if isinstance(value, str):
                entangled_fields.append(key)
                fields[key] = forms.CharField(
                    label=key,
                    widget=forms.TextInput if len(value) < 80 else forms.Textarea,
                    required=False,
                )
            elif isinstance(value, (list, tuple)):
                entangled_fields.append(key)
                fields[key] = CSValues(
                    label=key,
                    required=False,
                )
            elif isinstance(value, bool):
                entangled_fields.append(key)
                fields[key] = forms.BooleanField(
                    label=key,
                    required=False,
                )
            elif isinstance(value, decimal.Decimal):
                entangled_fields.append(key)
                fields[key] = forms.DecimalField(
                    label=key,
                    required=False,
                )

        fields["Meta"] = type(
            "Meta",
            (),
            {
                "model": FormEntry,
                "exclude": None,
                "entangled_fields": {"entry_data": entangled_fields},
                "untangled_fields": [
                    "form_name",
                    "form_user",
                ],
            },
        )
        return type("DynamicFormEntryForm", (EntangledModelForm,), fields)

    def get_admin_fieldsets(self):
        return (
            (
                None,
                {
                    "fields": (("form_name", "form_user"),),
                },
            ),
            (
                _("User-entered data"),
                {
                    "fields": tuple(
                        key
                        for key, value in self.entry_data.items()
                        if isinstance(value, (str, tuple, list, bool))
                    )
                },
            ),
        )

    def __str__(self):
        return f"{self.form_name} ({self.pk})"



from django.contrib import admin

from .models import FormEntry


@admin.register(FormEntry)
class FormEntryAdmin(admin.ModelAdmin):
    date_hierarchy = "entry_created_at"
    list_display = ("__str__", "form_user", "entry_created_at")
    list_filter = ("form_name", "form_user", "entry_created_at")
    readonly_fields = ["form_name", "form_user"]

    def has_add_permission(self, request):
        return False

    def get_form(self, request, obj=None, **kwargs):
        if obj:
            kwargs["form"] = obj.get_admin_form()
        return super().get_form(request, obj, **kwargs)

    def get_fieldsets(self, request, obj=None):
        if obj:
            return obj.get_admin_fieldsets()
        return super().get_fieldsets(request, obj)



# Generated by Django 3.2.13 on 2022-06-01 10:21

import re

import django.core.serializers.json
import django.core.validators
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import djangocms_form_builder.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Form',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_form_builder_form', serialize=False, to='cms.cmsplugin')),
                ('form_selection', models.CharField(blank=True, default='', max_length=256, verbose_name='Form')),
                ('form_name', models.CharField(blank=True, default='', max_length=256, validators=[django.core.validators.RegexValidator(re.compile('^[-a-zA-Z0-9_]+\\Z'), 'Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.', 'invalid')], verbose_name='Form name')),
                ('form_login_required', models.BooleanField(blank=True, default=False, help_text='To avoid issues with user experience use this type of form only on pages, which require login.', verbose_name='Login required to submit form')),
                ('form_unique', models.BooleanField(default=False, help_text='Requires "Login required" to be checked to work.', verbose_name='User can reopen form')),
                ('form_floating_labels', models.BooleanField(default=False, verbose_name='Floating labels')),
                ('form_spacing', models.CharField(max_length=16, verbose_name='Margin between fields')),
                ('form_actions', models.CharField(blank=True, max_length=1024, verbose_name='Actions to be taken after form submission')),
                ('attributes', djangocms_form_builder.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('captcha_widget', models.CharField(blank=True, choices=[('', '-----'), ('v2-checkbox', 'v2 checkbox'), ('v2-invisible', 'v2 invisible')], default='', help_text='Read more in the <a href="https://developers.google.com/recaptcha" target="_blank">documentation</a>.', max_length=16, verbose_name='reCaptcha widget')),
                ('captcha_requirement', models.DecimalField(decimal_places=2, default=0.5, help_text='Only for reCaptcha v3: Minimum score required to accept challenge.', max_digits=3, null=True, verbose_name='Minimum score requirement')),
                ('captcha_config', djangocms_form_builder.fields.AttributesField(blank=True, default=dict, help_text='The reCAPTCHA widget supports several <a href="https://developers.google.com/recaptcha/docs/display#render_param" target="_blank">data attributes</a> that customize the behaviour of the widget, such as <code>data-theme</code>, <code>data-size</code>. The reCAPTCHA api supports several <a href="https://developers.google.com/recaptcha/docs/display#javascript_resource_apijs_parameters" target="_blank">parameters</a>. Add these api parameters as attributes, e.g. <code>hl</code> to set the language.', verbose_name='Recaptcha configuration parameters')),
                ('tag_type', djangocms_form_builder.fields.TagTypeField(choices=[('div', 'div')], default='div', help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
            ],
            options={
                'verbose_name': 'Form',
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FormField',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='djangocms_form_builder_formfield', serialize=False, to='cms.cmsplugin')),
                ('ui_item', models.CharField(max_length=30)),
                ('tag_type', djangocms_form_builder.fields.TagTypeField(blank=True, choices=[('div', 'div'), ], default='div', help_text='Select the HTML tag to be used.', max_length=255, verbose_name='Tag type')),
                ('config', models.JSONField(default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
            ],
            options={
                'verbose_name': 'Form field item',
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FormEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('form_name', models.SlugField(verbose_name='Form')),
                ('entry_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('html_headers', models.JSONField(blank=True, default=dict)),
                ('entry_created_at', models.DateTimeField(auto_now_add=True)),
                ('entry_updated_at', models.DateTimeField(auto_now=True)),
                ('form_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Form entry',
                'verbose_name_plural': 'Form entries',
            },
        ),
        migrations.CreateModel(
            name='BooleanField',
            fields=[
            ],
            options={
                'verbose_name': 'Boolean field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='CharField',
            fields=[
            ],
            options={
                'verbose_name': 'Character field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='Choice',
            fields=[
            ],
            options={
                'verbose_name': 'Choice',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='DateField',
            fields=[
            ],
            options={
                'verbose_name': 'Date field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='DateTimeField',
            fields=[
            ],
            options={
                'verbose_name': 'Date field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='DecimalField',
            fields=[
            ],
            options={
                'verbose_name': 'Decimal field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='EmailField',
            fields=[
            ],
            options={
                'verbose_name': 'Email field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='IntegerField',
            fields=[
            ],
            options={
                'verbose_name': 'Integer field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='Select',
            fields=[
            ],
            options={
                'verbose_name': 'Select',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='SubmitButton',
            fields=[
            ],
            options={
                'verbose_name': 'Submit button',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='TextareaField',
            fields=[
            ],
            options={
                'verbose_name': 'Text field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='TimeField',
            fields=[
            ],
            options={
                'verbose_name': 'Date field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
        migrations.CreateModel(
            name='UrlField',
            fields=[
            ],
            options={
                'verbose_name': 'URL field',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('djangocms_form_builder.formfield',),
        ),
    ]



from cms.plugin_pool import plugin_pool
from django.conf import settings

if 'djangocms_frontend.contrib.frontend_forms' in settings.INSTALLED_APPS:
    from djangocms_frontend.contrib.frontend_forms.cms_plugins import (
        BooleanFieldPlugin,
        CharFieldPlugin,
        ChoicePlugin,
        DateFieldPlugin,
        DateTimeFieldPlugin,
        DecimalFieldPlugin,
        EmailFieldPlugin,
        FormPlugin,
        IntegerFieldPlugin,
        SelectPlugin,
        TextareaPlugin,
        TimeFieldPlugin,
        URLFieldPlugin,
    )

    for plugin in (
        FormPlugin,
        BooleanFieldPlugin,
        CharFieldPlugin,
        ChoicePlugin,
        DateFieldPlugin,
        DateTimeFieldPlugin,
        DecimalFieldPlugin,
        EmailFieldPlugin,
        IntegerFieldPlugin,
        SelectPlugin,
        TextareaPlugin,
        TimeFieldPlugin,
        URLFieldPlugin,
    ):
        plugin_pool.unregister_plugin(plugin)



from django import template
from django.apps import apps
from django.template.loader import render_to_string
from django.utils.html import mark_safe

from .. import constants, recaptcha
from ..helpers import get_option
from ..settings import FORM_TEMPLATE

register = template.Library()
attr_dict = constants.attr_dict
default_attr = constants.default_attr


if apps.is_installed("crispy_forms"):
    from crispy_forms.helper import FormHelper
    from crispy_forms.utils import render_crispy_form as render_form_implementation

    crispy_forms_installed = True
else:
    crispy_forms_installed = False

    def render_form_implementation(form, helper=None, context=None):
        return str(form)

    class FormHelper:
        def __init__(self, form):
            self.form = form


@register.filter
def add_placeholder(form):
    """Adds placeholder based on a form field's title"""
    for field_name, _ in form.fields.items():
        form.fields[field_name].widget.attrs["placeholder"] = form.fields[
            field_name
        ].label
    return form


@register.simple_tag()
def render_form(form, **kwargs):
    """Renders form either with crispy_forms if installed and form has helper or with
    django-formset's means"""
    if crispy_forms_installed:
        helper = kwargs.pop("helper", None) or getattr(form, "helper", None)
        if helper is None and get_option(form, "crispy_form"):
            helper = FormHelper(form=form)
        if helper is not None:
            helper.form_tag = False
            helper.disable_csrf = True
            return mark_safe(render_form_implementation(form, helper, None))
    template = kwargs.pop("template", FORM_TEMPLATE)
    return render_to_string(template, {"form": form, **kwargs})


def get_bound_field(form, formfield):
    for field in form.visible_fields():
        if field.name == formfield:
            return field
    return None


def attrs_for_widget(widget, item, additional_classes=None):
    if widget.__class__.__name__ in constants.attr_dict:
        cls = attr_dict[widget.__class__.__name__].get(item, default_attr[item])
    else:
        cls = default_attr[item]
    if cls:
        if additional_classes:
            cls += " " + additional_classes
    else:
        cls = additional_classes or ""
    return {"class": cls}


@register.simple_tag(takes_context=False)
def render_widget(form, form_field, **kwargs):
    field = get_bound_field(form, form_field)
    if field is None:
        return ""
    floating_labels = get_option(form, "floating_labels")
    if field.field.widget.attrs.pop("no_field_sep", False):
        field_sep = ""
    else:
        field_sep = get_option(form, "field_sep", constants.DEFAULT_FIELD_SEP)
    widget_attr = kwargs
    if form.is_bound:
        add_classes = "is_invalid" if field.errors else "is_valid"
    else:
        add_classes = None
    widget_attr.update(
        attrs_for_widget(field.field.widget, "input", additional_classes=add_classes)
    )
    label_attr = attrs_for_widget(field.field.widget, "label")
    if field.help_text:
        widget_attr.update({"aria-describedby": f"hints_{field.id_for_label}"})
        help_text = f'<div id="hints_{field.id_for_label}" class="form-text">{field.help_text}</div>'
    else:
        help_text = ""
    input_type = getattr(field.field.widget, "input_type", None)
    if floating_labels:
        widget_attr.setdefault("placeholder", "-")
        if input_type not in ("checkbox", "radio"):
            field_sep += " form-floating"  # TODO: Only true for Bootstrap5
    div_attrs = attrs_for_widget(field.field.widget, "div", field_sep)
    div_attrs = " ".join([f'{key}="{value}"' for key, value in div_attrs.items()])
    grp_attrs = attrs_for_widget(field.field.widget, "group")
    errors = "".join(
        f'<div class="invalid-feedback">{error}</div>' for error in field.errors
    )
    if field.field.widget.template_name.rsplit("/", 1)[-1] in (
        "radio.html",
        "checkbox_select.html",
    ):
        """For multi-valued widgets use own templates to ensure classes appear at the right nesting"""
        field.field.widget.template_name = (
            "djangocms_form_builder/widgets/mutliple_input.html"
        )
        field.field.widget.option_template_name = (
            "djangocms_form_builder/widgets/input_option.html"
        )
        widget_attr["label_class"] = label_attr.pop(
            "class", None
        )  # pass through label classes
        widget_attr["div_class"] = grp_attrs.pop(
            "class", None
        )  # pass through div classes
        input_first = False
    else:
        input_first = (
            floating_labels
            or input_type == "checkbox"
            or input_type == "select"
            and floating_labels
        )
    widget = field.as_widget(attrs=widget_attr)
    label = field.label_tag(attrs=label_attr) if field.label else ""
    if input_first:
        render = f"<div {div_attrs}>{widget}{label}{errors}{help_text}</div>"
    else:
        render = f"<div {div_attrs}>{label}{widget}{errors}{help_text}</div>"
    return mark_safe(render)


@register.simple_tag(takes_context=False)
def render_recaptcha_widget(form):
    if recaptcha.installed:
        return render_widget(form, recaptcha.field_name)
    return ""


@register.filter_function
def get_fieldset(form):
    """returns the fieldsets of a form if available or generates a fieldset as a
    list of all fields"""
    if hasattr(form, "get_fieldsets") and callable(form.get_fieldsets):
        return form.get_fieldsets()
    elif hasattr(form, "Meta") and hasattr(form.Meta, "fieldsets"):
        return form.Meta.fieldsets
    return ((None, {"fields": [field.name for field in form.visible_fields()]}),)



from django.urls import path

from . import views

app_name = "djangocms_form_builder"

urlpatterns = [
    path("f<form_id>", views.AjaxView.as_view(), name="ajaxformbuilder"),
    path(
        "<int:instance_id>/<path:parameter>",
        views.AjaxView.as_view(),
        name="ajaxview",
    ),
    path(
        "<int:instance_id>",
        views.AjaxView.as_view(),
        name="ajaxview",
    ),
]



# Generated by Django 4.0 on 2022-10-19 10:08

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0022_auto_20180620_1551'),
        ('djangocms_form_builder', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='form',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='%(app_label)s_%(class)s', serialize=False, to='cms.cmsplugin'),
        ),
        migrations.AlterField(
            model_name='formfield',
            name='cmsplugin_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='%(app_label)s_%(class)s', serialize=False, to='cms.cmsplugin'),
        ),
    ]
